import {
  select,
  csv,
  scaleLinear,
  scalePoint,
  extent,
  axisBottom,
  axisLeft,
} from 'd3';
import { dropdownMenu } from './dropdownMenu';
import { horizontalChart } from './horizontalChart';
import { scatterPlot } from './scatterPlot';

const svg = select('svg');

const width = +svg.attr('width');
const height = +svg.attr('height');

let data;
let typeCounts = {};
let types;
const allTypes = {
  normal: '#AAB09F',
  fighting: '#C22E28',
  flying: '#7DA6DE',
  poison: '#A33EA1',
  ground: '#927D44',
  rock: '#B2A061',
  bug: '#94BC4A',
  ghost: '#846AB6',
  steel: '#89A1B0',
  fire: '#EE8130',
  water: '#6390F0',
  grass: '#7AC74C',
  electric: '#E5C531',
  psychic: '#E5709B',
  ice: '#96D9D6',
  dragon: '#6A7BAF',
  dark: '#736C75',
  fairy: '#E397D1',
};
let selectedType = null;

const render = () => {
  // Create a table for types and type effectiveness
  const table = svg
    .append('foreignObject')
    .attr('transform', `translate(400,0)`)
    .attr('width', 700) // Set the width of the table
    .attr('height', height)
    .append('xhtml:table')
    .style('border-collapse', 'collapse');
    

  // Add header row
  const headerRow = table.append('thead').append('tr');
  headerRow.append('th').text('Types');
  Object.keys(allTypes).forEach((type) => {
    headerRow
      .append('th')
      .text(type.substring(0, 3).toUpperCase())
      .style('background-color', allTypes[type]) // Set background color
      .style('color', 'white');
  });

  // Add rows for type effectiveness
  const tbody = table.append('tbody');
  Object.keys(allTypes).forEach((type1) => {
    const row = tbody
      .append('tr')
      .attr('class', `row-${type1}`);
    row
      .append('td')
      .text(type1)
      .style('background-color', allTypes[type1]) // Set background color
      .style('color', 'white');

    Object.keys(allTypes).forEach((type2) => {
      const effectiveness = getTypeEffectiveness(
        type1,
        type2,
      );
      const cell = row
        .append('td')
        .attr(
          'class',
          effectiveness != 1 ? 'filled' : 'blank',
        )
        .style(
          'background-color',
          effectiveness > 1
            ? 'green'
            : effectiveness < 0.4
              ? 'red'
              : effectiveness < 0.7
                ? 'orange'
                : 'transparent',
        )
        .on('mouseover', function () {
          const [x, y] = d3.mouse(this);
          // Show tooltip dynamically on mouseover
          const tooltip = select(this)
            .append('div')
            .classed('tooltip', true);
          tooltip
            .html(
              `${type2} ->${type1} = ${
                effectiveness > 1
                  ? 'super effective'
                  : effectiveness < 0.4
                    ? 'not effective'
                    : effectiveness < 0.7
                      ? 'not very effective'
                      : 'normal effectiveness'
              }`,
            )
            .style(
              'transform',
              `translate(${x}px, ${y}px)`,
            );
        })
        .on('mouseout', function () {
          // Remove the tooltip on mouseout
          select(this).select('.tooltip').remove();
        });
    });
  });

  // Call horizontalChart function
  svg.call(horizontalChart, {
    margin: {
      top: 30,
      right: 800,
      bottom: 70,
      left: 70,
    },
    width,
    height,
    data: typeCounts,
    attributes: allTypes,
    xAxisLabel: 'Count',
    yAxisLabel: 'Types',
    onTypeSelect: (type) => {
      selectedType = type;
      updateTableStyles();
    },
  });
  updateTableStyles();
};

const updateTableStyles = () => {
  // Update row styles based on the selected type
  if (!Array.isArray(selectedType)) {
    svg
      .selectAll(`.row-${selectedType}`)
      .selectAll('.blank')
      .style('background-color', '#8888FF');
  } else {
    svg.selectAll('.selected-type-text').remove(); // Remove existing text
    svg
      .append('text')
      .attr('class', 'selected-type-text')
      .attr('x', 450)
      .attr('y', height - 50)
      .attr('text-anchor', 'middle')
      .text(`Selected Types: `);
    // Reset all rows to the default background color
    svg
      .selectAll('.blank')
      .style('background-color', 'white');

    // Set the background color for the selected rows
    selectedType.forEach((type, i) => {
      svg
        .selectAll(`.row-${type}`)
        .selectAll('.blank')
        .style('background-color', '#8888FF');
      svg
        .append('text')
        .attr('class', 'selected-type-text')
        .attr('x', 550 + 60 * i) // Adjust the x-coordinate as needed
        .attr('y', height - 50) // Adjust the y-coordinate as needed
        .attr('text-anchor', 'middle')
        .text(`${type}`);
    });
  }
};

function darkenColor(color, factor) {
  return d3.color(color).darker(factor).toString();
}

// Helper function to get type effectiveness (replace with your own logic)
const getTypeEffectiveness = (type1, type2) => {
  // Replace this logic with your own type effectiveness calculations
  // For example, you can use a data structure or function to determine effectiveness
  const typeEffectiveness = {};
  types.forEach((defender) => {
    typeEffectiveness[defender] = {};
    types.forEach((attacker) => {
      typeEffectiveness[defender][attacker] = 1.0;
    });
  });
  typeEffectiveness['normal']['fighting'] = 1.6;
  typeEffectiveness['normal']['ghost'] = 0.39;
  typeEffectiveness['fighting']['flying'] = 1.6;
  typeEffectiveness['fighting']['psychic'] = 1.6;
  typeEffectiveness['fighting']['fairy'] = 1.6;
  typeEffectiveness['fighting']['rock'] = 0.63;
  typeEffectiveness['fighting']['bug'] = 0.63;
  typeEffectiveness['fighting']['dark'] = 0.63;
  typeEffectiveness['flying']['rock'] = 1.6;
  typeEffectiveness['flying']['electric'] = 1.6;
  typeEffectiveness['flying']['ice'] = 1.6;
  typeEffectiveness['flying']['fighting'] = 0.63;
  typeEffectiveness['flying']['grass'] = 0.63;
  typeEffectiveness['flying']['bug'] = 0.63;
  typeEffectiveness['flying']['ground'] = 0.39;
  typeEffectiveness['poison']['ground'] = 1.6;
  typeEffectiveness['poison']['psychic'] = 1.6;
  typeEffectiveness['poison']['fighting'] = 0.63;
  typeEffectiveness['poison']['poison'] = 0.63;
  typeEffectiveness['poison']['bug'] = 0.63;
  typeEffectiveness['poison']['grass'] = 0.63;
  typeEffectiveness['poison']['fairy'] = 0.63;
  typeEffectiveness['ground']['water'] = 1.6;
  typeEffectiveness['ground']['grass'] = 1.6;
  typeEffectiveness['ground']['ice'] = 1.6;
  return typeEffectiveness[type1][type2];
};

csv('pokemon_1500.csv').then((loadedData) => {
  data = loadedData;
  types = Array.from(
    new Set(
      data.flatMap((d) => [d['Type 1'], d['Type 2']]),
    ),
  );
  types.forEach((type) => {
    if (type != 'none') {
      typeCounts[type] = data.filter(
        (d) => d['Type 1'] === type || d['Type 2'] === type,
      ).length;
    }
  });
  console.log(typeCounts);
  data.forEach((d) => {});
  render();
});
